module ActiveResource::Hel

  class FailedAuthentication < RuntimeError; end

  #
  # Session context for the Hel Connection
  #
  class Session

    class Invalid < RuntimeError; end

    AUTH_METHOD          = '/ygg/session/authenticate_by_fqda_and_password'
    TRY_AUTH_METHOD      = '/ygg/session/would_authenticate_by_fqda_and_password'
    SESSION_CHECK_METHOD = '/ygg/session/check'

    DEFAULT_TIMEOUT = 30

    attr_reader :site
    attr_reader :data
    attr_reader :token
    attr_reader :identity
    attr_reader :status
    attr_reader :authenticated
    attr_reader :message
    attr_reader :reason

    #debug
    attr_writer :token

    def initialize(options = {})
      @site = options[:site]
      @identity = options[:identity]
      @secret = options[:secret]
      @ssl_options = options[:ssl_options]

      if options[:token]
        @token = options[:token]
        @authenticated = true
      end

      @timeout = options.fetch(:timeout, DEFAULT_TIMEOUT)
      @connection = ActiveResource::Connection.new(@site)
      @connection.ssl_options = @ssl_options

      raise Invalid, 'Missing identity when instantiating a session' if identity.blank?
    end


    def initialize_from_data(auth_data)
      @data = auth_data
      @token, @status = @data['id'], @data['status']

      @authenticated = @data.fetch('authenticated', false)
      @message = @data.fetch('msg', 'No Message Given')
      @reason  = @data.fetch('reason', 'no_reason')

      if authenticated?
        @identity = @data.fetch('auth_identity', @identity)
      end

      self
    end

    def authenticated?
      authenticated && !token.blank?
    end

    def active_identity
      if authenticated?
        @active_identity ||= Ygg::Core::Identity.find_by_qualified_for_authentication(@identity)
      end
    rescue ActiveResource::Hel::FailedAuthentication
      @active_identity = nil
    end

    def with_authentication(&block)
      authenticate! unless authenticated?
      do_request &block
    rescue ActiveResource::UnauthorizedAccess => e
      begin
        authenticate!
      rescue Invalid
        raise e
      end
      do_request &block
    end

    def authenticate!
      raise Invalid, 'Invalid session credentials' if identity.blank? || @secret.blank?
      login_data = {:fqda => identity, :password => @secret}
      initialize_from_data post(AUTH_METHOD, login_data)
      authenticated? || raise(ActiveResource::Hel::FailedAuthentication, "#{message} [#{reason}]")
    end

    def try_authenticate(data)
      post(TRY_AUTH_METHOD, data)
    end

    def would_authenticate?(data)
      res = try_authenticate(data)

      if res.has_key?('authenticated') && res['authenticated']
        return res
      else
        false
      end
    rescue
      false
    end

    def authentication_headers
      {'Authorization' => "Bearer #{token}"}
    end

    def post(path, object, headers = {})
      request_body = object.to_json
      res = @connection.post(path, request_body, headers.merge('Accept' => 'application/json'))
      ActiveSupport::JSON.decode(res.body)
    end

    def do_request(&block)
      block.call authentication_headers if block_given?
    end

  end
end
