module Ygg
  class Session < ActiveResource::Hel::Resource
    self.cached = false

    class Exception < StandardError; end

    class << self

      #
      # Gets you a session on behalf of the given identity
      #
      def new_for_identity(identity, attributes = {})
        identity = identity.qualified if identity.is_a? Ygg::Core::Identity
        session = instantiate_record(proxy_authenticate_by_fqda(attributes.merge(:fqda => identity)))

        raise(Exception, session.msg) unless check_reported_valid(session)
        session
      end

      #
      # checks the given session for validity
      # takes session || token
      #
      def check_validity(given_session)

        given_session = given_session.token if given_session.respond_to?(:token)
        check(:session_id => given_session)
      end


      # API methods
      # --------------------------------------

      # Responds with an authentication token if the user would authenticate using a fqda/password pair
      #
      # If the session-id is not given in a X-Ygg-Session-Id header, a new session is created at the same time.
      # The specified or newly created session's id is included in the reply's header X-Ygg-Session-Id.
      #
      # The accepted parameters are:
      # [:fqda]           FQDA (fully qualified domain address).
      # [:username]       FQDA (fully qualified domain address), accepted when [:fqda] is not specified as a way to
      #                   support browsers' password managers.
      # [:password]       The cleartext password.
      #
      # The reply contains the authentication token
      #
      def would_authenticate_by_fqda_and_password(params)
        post 'would_authenticate_by_fqda_and_password', {}, params
      end

      # Attempts to authenticate a session by using a fqda/password pair for a third party.
      #
      # Session ID is accepted in the request body only, along with HTTP environment data.
      #
      # The accepted parameters are:
      # [:fqda]           FQDA (fully qualified domain address).
      # [:password]       The cleartext password.
      #
      # [:remote_addr]      HTTP parameters
      # [:remote_port]
      # [:x_forwarded_for]
      # [:via]
      # [:server_addr]
      # [:server_port]
      # [:server_name]
      # [:referer]
      # [:user_agent]
      # [:request_uri]
      #
      # The reply contains the session object.
      # In addition to the session's usual attributes two attributes are included to describe the authentication result:
      #
      # [:msg]            Human-readable explanation of the result
      # [:reason]         Can be either :ok or :wrong_credentials
      #
      def proxy_authenticate_by_fqda(params)
        post 'proxy_authenticate_by_fqda', {}, params
      end

      # Authenticate a session as third party
      #
      # Session ID is accepted in the request body only, along with HTTP environment data.
      #
      # The accepted parameters are:
      # [:fqda]           FQDA (fully qualified domain address).
      # [:remote_addr]      HTTP parameters
      # [:remote_port]
      # [:x_forwarded_for]
      # [:via]
      # [:server_addr]
      # [:server_port]
      # [:server_name]
      # [:referer]
      # [:user_agent]
      # [:request_uri]
      #
      # The reply contains the session object.
      # In addition to the session's usual attributes two attributes are included to describe the authentication result:
      #
      # [:msg]            Human-readable explanation of the result
      # [:reason]         Can be either :ok or :wrong_credentials
      #
      def proxy_authenticate_by_fqda_and_password(params)
        post 'proxy_authenticate_by_fqda_and_password', {}, params
      end

      #
      def check(params = nil)
       # if params
          post 'check', {}, params
       # else
       #   get 'check'
       # end
      end


      #######################################################


      # collection name, since hel session_s_ are on the 'session' controller
      def collection_name
       "session"
      end

      #??
      def find(*arguments)
        session = check(:session_id => arguments.first)
        instantiate_record(session)
      end

      private

      def check_reported_valid(session)
        session.authenticated?
      end
    end

    #########################


    def identity
      Ygg::Core::Identity.find_by_qualified(auth_identity) if auth_identity
    end

    def token
      id
    end

    def check
      self.class.check_validity(self)
    end

  end
end
